@@ -190,6 +190,8 @@ def imagen_sample_in_chunks(fn):
     return inner
 
 class ImagenTrainer(nn.Module):
+    initted = False
+
     def __init__(
         self,
         imagen,
@@ -212,6 +214,8 @@ class ImagenTrainer(nn.Module):
         **kwargs
     ):
         super().__init__()
+        assert not ImagenTrainer.initted, 'ImagenTrainer can only be initialized once per process - for the sake of distributed training, you will now have to create a separate script to train each unet (or a script that accepts unet number as an argument)'
+        ImagenTrainer.initted = True
 
         assert isinstance(imagen, (Imagen, ElucidatedImagen))
         ema_kwargs, kwargs = groupby_prefix_and_trim('ema_', kwargs)
@@ -243,9 +247,6 @@ class ImagenTrainer(nn.Module):
 
         self.ema_unet_being_trained_index = -1 # keeps track of which ema unet is being trained on
 
-        self.only_train_unet_number = only_train_unet_number
-        self.validate_and_set_unet_being_trained(only_train_unet_number)
-
         # data related functions
 
         self.train_dl_iter = None
@@ -308,6 +309,11 @@ class ImagenTrainer(nn.Module):
         self.imagen.to(self.device)
         self.to(self.device)
 
+        # only allowing training for unet
+
+        self.only_train_unet_number = only_train_unet_number
+        self.validate_and_set_unet_being_trained(only_train_unet_number)
+
     # computed values
 
     @property
@@ -327,8 +333,8 @@ class ImagenTrainer(nn.Module):
         return self.accelerator.is_local_main_process
 
     @property
-    def unwrapped_imagen(self):
-        return self.accelerator.unwrap_model(self.imagen)
+    def unwrapped_unet(self):
+        return self.accelerator.unwrap_model(self.unet_being_trained)
 
     # function for allowing only one unet from being trained at a time
 
